@@ -1,4 +1,5 @@
 use activitystreams::checked::CheckError;
+use actix_rt::task::JoinError;
 use actix_web::{
     error::{BlockingError, ResponseError},
     http::StatusCode,
@@ -193,6 +194,12 @@ impl From<BlockingError> for ErrorKind {
     }
 }
 
+impl From<JoinError> for ErrorKind {
+    fn from(_: JoinError) -> Self {
+        ErrorKind::Canceled
+    }
+}
+
 impl From<Infallible> for ErrorKind {
     fn from(i: Infallible) -> Self {
         match i {}
